/*
 *  Copyright 2002 Adrian Thurston <adriant@ragel.ca>
 */

/*  This file is part of Aapl.
 *
 *  Aapl is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  Aapl is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with Aapl; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
 */

#ifndef _AAPL_AVLIMELKEY_H
#define _AAPL_AVLIMELKEY_H

#include "compare.h"
#include "dlistmel.h"

/**
 * \addtogroup avltree 
 * @{
 */

/**
 * \class AvliMelKey
 * \brief Linked AVL tree for nodes appearing in multiple trees with different keys.
 *
 * AvliMelKey is the same as AvlMel with the added feature of being able to use
 * different keys for different trees. The location of the getKey routine is
 * given to AvliMelKey template in the same was the location of the AvlNode is
 * given.
 *
 * AvliMelKey implicitly connects nodes with linked list pointers, allowing the
 * nodes to be walked in order using next and previous pointers.
 *
 * AvliMelKey does not explicitly manage memory for nodes. Nodes inserted into
 * the tree are typically allocated by the user. Since nodes can have static
 * allocation, the tree does not assume ownership of the nodes. The destructor
 * will not delete nodes. A deep copy will cause existing elements to be
 * abandoned.
 *
 * \include ex_avlimelkey.cpp
 */

/*@}*/

#define BASENODE(name) BaseNode::name
#define BASEKEY(name) BaseKey::name
#define BASECOMPARE(name) BaseKey::name
#define BASELIST DListMel< Node, BaseNode >
#define AVLMEL_TEMPDEF class Node, class Key, class BaseNode, class BaseKey
#define AVLMEL_TEMPUSE Node, Key, BaseNode, BaseKey
#define AvlTree AvliMelKey
#define WALKABLE

#include "avlcommon.h"

#undef BASENODE
#undef BASEKEY
#undef BASECOMPARE
#undef BASELIST
#undef AVLMEL_TEMPDEF
#undef AVLMEL_TEMPUSE
#undef AvlTree
#undef WALKABLE

#endif /* _AAPL_AVLIMELKEY_H */
