#
#   Copyright 2002 Adrian Thurston <adriant@ragel.ca>
#

#   This file is part of Ragel.
#
#   Ragel is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   Ragel is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with Ragel; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 


#******************************************************************************
# This is a common set of rules for all tests. Variables affect how this rules
# file behaves. Those variables are:
#
# LANG:    Specify the language of the generated fsm. Can be either c or c++.
# TEST:    The name of the ragel file that the test is in.
#
#******************************************************************************

#
# The following variables affect what tests are compiled.
#

ifndef CODEGEN_OPTS
	# Code generation options. There should be at least one option specified.
	CODEGEN_OPTS = -T0 -T1 -S0 -G0 -G1 -G2
endif

ifndef COMPACT_OPTS
	# State compaction options. There should be at least one option specified.
	COMPACT_OPTS = -n -a -b -j -k -ae -be -je -ke
endif

# The location of executables used.
RAGEL = ../../ragel
RLCOMPILE = ../rlcompile
RUNTESTS = ../runtests

#******************************************************************************

# C code
ifeq ($(LANG),c)
LANGOPT = -c
CC = gcc
CSUFFIX = c
endif

# C++ code
ifeq ($(LANG),c++)
LANGOPT = -C
CC = g++
CSUFFIX = cpp
endif

# Stem of the test to run.
TEST_STEM=$(TEST:%.rl=%)

# Make the names of the tests.
STEM1 = $(foreach opt, $(CODEGEN_OPTS:-%=%), $(TEST_STEM:%=$(opt)%))
STEM2 = $(foreach opt, $(COMPACT_OPTS:-%=%), $(STEM1:%=$(opt)%))

# Make all binary targets and c source targets.
ALLSRCS = $(STEM2:%=%.$(CSUFFIX))
ALLTARGS = $(STEM2:%=%.bin)


# Rules
all: $(ALLTARGS)

clean:
	rm -f *.$(CSUFFIX) *.bin

distclean: clean

# For all compiled versions, run all test inputs through diff
# the output with the expected results.
test: $(ALLTARGS)
	@$(RUNTESTS) $(ALLTARGS)

# Compile rules
%.bin: %.c
	$(CC) $(CFLAGS) $(PFLAGS) -o $@ $<

%.bin: %.cpp
	$(CC) $(CFLAGS) $(PFLAGS) -o $@ $<

# Ragel compile.
$(ALLSRCS): $(TEST) $(RAGEL)
	@$(RLCOMPILE) $(LANGOPT) $(@:%$(TEST_STEM).$(CSUFFIX)=%) $@ $(TEST)
